{****************************************************************
 * RoboPi Serial Protocol Handler
 *
 * Copyright 2014 William Henning
 *
 * Version 0.81 June 2, 2014
 *
 * - added readDistance()
 * - improved pwm/servo accuracy to 4.96us timing grain
 *
 * Version 0.70 - initial release
 *
 * RoboPi product page:
 *
 * http://www.mikronauts.com/raspberry-pi/robopi/
 *
 * RoboPi discussion threads:
 *
 * http://forums.parallax.com/showthread.php/153275
 *
 * http://www.raspberrypi.org/forum/viewtopic.php?f=37&t=68416
 *
 ****************************************************************}

CON

    _clkmode = xtal1 + pll16x
'    _xinfreq = 5_000_000
    _xinfreq = 6_250_000

    SOH = 1
    EOT = 4

    GETINFO     = 0             ' returns product string

    ' commands 1-10 are RESERVED

    READMODE    = 11            ' write pin mode
    WRITEMODE   = 12            ' write pin mode

    DIGREAD     = 13            ' digital read
    DIGWRITE    = 14            ' digital write

    ANREAD      = 15            ' analog read 10 bit result
    ANREADRAW   = 16            ' analog read at raw resolution
    ANWRITE     = 17            ' analog write 0..255 used for PWM

    SERVOWRITE  = 18            ' write a servo 0..20000us, normally 500-2500us
    SERVOREAD   = 19            ' read last value written to servo

    READDIST    = 20            ' read distance sensor

    ' commands 20-255 are RESERVED

OBJ
    
  robot:  "RoboPiObj_v0_81"     ' starts two cogs
    ser:  "FullDuplexSerial"    ' starts one cog

VAR

    byte    check1, check2, buffer[256]

PUB main|i,j,cmd,len,me,analog

    robot.start

    ser.start(31,30,0,115200)

    robot.delay(100)

    repeat

      repeat while GetPacket(@me,@cmd,@buffer[0],@len) == 0

      case cmd

        GETINFO:     ' returns product string
            SendPacket(me,GETINFO,@product_info,strsize(@product_info))

        READMODE:   ' read pin mode
            buffer[1] := robot.readMode(buffer[0])
            SendPacket(me,cmd,@buffer[0],len+1)

        WRITEMODE:   ' write pin mode
            robot.pinMode(buffer[0],buffer[1])
            SendPacket(me,cmd,@buffer[0],len)

        DIGREAD:     ' digital read
            buffer[1] := robot.digitalRead(buffer[0])
            SendPacket(me,cmd,@buffer[0],len+1)

        DIGWRITE:    ' digital write
            robot.digitalWrite(buffer[0],buffer[1])
            SendPacket(me,cmd,@buffer[0],len)

        ANREAD:      ' analog read 10 bit result
            analog := robot.analogRead(buffer[0])
            buffer[1] := analog & 255
            buffer[2] := analog >> 8
            SendPacket(me,cmd,@buffer[0],len+2)

        ANREADRAW:   ' analog read at raw resolution
            analog := robot.analogReadRaw(buffer[0])
            buffer[1] := analog & 255
            buffer[2] := analog >> 8
            SendPacket(me,cmd,@buffer[0],len+2)

        ANWRITE:     ' analog write 0..255 used for PWM
            robot.analogWrite(buffer[0],buffer[1])
            SendPacket(me,cmd,@buffer[0],len)

        SERVOWRITE:  ' write a servo 0..20000us, normally 500-2500us
            robot.servoWrite(buffer[0],buffer[1] | (buffer[2]<<8))
            SendPacket(me,cmd,@buffer[0],len)

        SERVOREAD:   ' read last value written to servo
            analog := robot.servoRead(buffer[0])
            buffer[1] := analog & 255
            buffer[2] := analog >> 8
            SendPacket(me,cmd,@buffer[0],len)

        READDIST:    ' read distance sensor, result is i mm
            analog := robot.readDistance(buffer[0])
            buffer[1] := analog & 255
            buffer[2] := analog >> 8
            SendPacket(me,cmd,@buffer[0],len+2)

        other: ' echo back what we received, indicate error by or-ing $80 with cmd
            SendPacket(me,$80 | cmd,@buffer[0],len)


PUB SendPacket(addr, cmd, ptr, size)|sum,byt
    sum := size + cmd + addr
    ser.Tx(SOH)
    ser.Tx(addr)
    ser.Tx(cmd)
    ser.Tx(size)
    repeat size
        byt := byte[ptr++]
        sum += byt
        ser.Tx(byt)
    ser.Tx(sum)
    ser.Tx(EOT)

PUB GetPacket(addr, cmdp, ptr, size):status|ch,sum,len,byt,chk,cmd,me
    repeat until (ser.rx == SOH)
    me  := ser.rx
    cmd := ser.rx
    len := ser.rx
    sum := me + cmd + len
    repeat len
      byt := ser.rx
      byte[ptr++] := byt
      sum += byt
    chk := ser.rx
    long[size] := len
    long[cmdp] := cmd
    long[addr] := me
    return chk == (sum & 255)

dat

product_info  byte "RoboPi http://Mikronauts.com API v0.70",0



