{****************************************************************
 * RoboPiOb: Wiring compatible object
 *
 * Copyright 2014 William Henning
 *
 * Version 0.81 - June 2, 2014
 *
 * - added readDistance()
 * - improved pwm/servo accuracy to 4.96us timing grain
 *
 * You can find RoboPi at http://Mikronauts.com
 *
 * I will be adding more capabilites (and error checking) as
 * time permits.
 *
 * RoboPi product page:
 *
 * http://www.mikronauts.com/raspberry-pi/robopi/
 *
 * RoboPi discussion threads:
 *
 * http://forums.parallax.com/showthread.php/153275
 *
 * http://www.raspberrypi.org/forum/viewtopic.php?f=37&t=68416
 *
 ****************************************************************
 * RoboPi currently uses the following objects:
 ****************************************************************
 *
 * "ADC_INPUT_DRIVER" - MCP3208 driver
 *  ----------------------------------
 *
 *  ADC Input Driver v2.0
 *  Supports 8- to 16-bit, up to 8 channels
 *  Designed for the MCP3X0X series of ADCs
 *  Author: Brandon Nimon
 *  Created: 16 July, 2009
 *  Copyright (c) 2009 Parallax, Inc.
 *  MIT Licensed, see object source for details
 *
 * "PWM_32_v4_2.spin" - servo/pwm modified version of Beau's object
 *  ------------------------------------
 *
 *  PWM Object V4.0
 *  Author: Beau Schwabe
 *  Copyright (c) 2009 Parallax, Inc.
 *  MIT Licensed, see object source for details
 *
 ****************************************************************}

CON

  HIGH   = 1
  LOW    = 0

  MAXPIN = 23

  INPUT  = 0
  OUTPUT = 1

  PWM    = $10
  SERVO  = $20

OBJ

  adcobj   : "ADC_INPUT_DRIVER"
  pwmobj   : "PWM_32_v4_2.spin"
  pingobj  : "UltrasoundSR04_120612a.spin"

VAR

  long  inputs, outputs

  long  one_ms, one_us

  word  servo_val[32]

  byte  pin_mode[32]

PUB start|i

  one_ms := clkfreq/1000
  one_us := one_ms/1000

  repeat i from 0 to MAXPIN
    pin_mode[i] := INPUT
    servo_val   := 0

  adcobj.start(25, 24, 26, 27, 8, 8, 12, 1)

  pwmobj.start

'-------------------------------------------------
' Arduino Compatibility
'-------------------------------------------------

pub pinMode(pin, mode)
  if (pin_mode[pin] == PWM) OR (pin_mode[pin] == SERVO)
    pwmobj.StateMode(pin,0)
  pin_mode[pin] := mode
  if (mode==OUTPUT)
    dira[pin]~~
  else
    dira[pin]~

pub readMode(pin)
  return pin_mode[pin]

pub digitalRead(pin)
  if (pin_mode[pin] == PWM) OR (pin_mode[pin] == SERVO)
    pwmobj.StateMode(pin,0)
    pin_mode[pin] := INPUT
  if (pin =< MAXPIN)
    return ina[pin]

pub digitalWrite(pin, value)
  if (pin_mode[pin] == PWM) OR (pin_mode[pin] == SERVO)
    pwmobj.StateMode(pin,0)
    pin_mode[pin] := OUTPUT
    outa[pin]:=0
  if pin_mode[pin] == OUTPUT
    outa[pin] := value

pub analogRead(pin)  ' returns 0..1023
  return adcobj.getval(pin)>>2

pub analogReadRaw(pin) ' returns value to ADC's max resolution, does not scale
  return adcobj.getval(pin)

pub analogWrite(pin, value) ' 0..255
  pin_mode[pin] := PWM
  pwmobj.pwm(pin,value,255-value)

pub servoWrite(pin,value)
  servo_val[pin] := value
  pwmobj.servo(pin,value)

pub servoRead(pin)
  return servo_val[pin]

pub readDistance(pech)
  return pingobj.Millimeters(pech,pech)

pub delay(ms)
  waitcnt(cnt+ms*one_ms)

pub delayMicroseconds(us)|u
  u := cnt + us * one_us
  if us > 10
    waitcnt(u)


    
